package assinante;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import br.usp.ime.pictware.business.assinante.bo.Assinante;

public class TestAssinanteORM {

	private EntityManager em;
	
	@Before
	public void setUp() throws Exception {
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("PictWareTestPU");
        em = emf.createEntityManager();
	}

	@After
	public void tearDown() throws Exception {
	}
	
	@Test
	public void cadastrarAssinante(){
		
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
				
			//Constri um assinante
			Assinante assinante = new Assinante();
			assinante.setLogin("x@x.com");
			assinante.setSenha("12345");
			assinante.setNome("Joo Cruz");
			assinante.setApelido("Joo");
			
			//Persiste o assinante
			em.persist(assinante);			
			tx.commit();
			
			Assert.assertNotNull(assinante.getId());
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}
	}
	
	@Test
	public void cadastrarRemoverAssinante(){
        
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
				
			/** Cadastra Assinante */
			
			//Constri um assinante
			Assinante assinante = new Assinante();
			assinante.setLogin("maria@terra.com.br");
			assinante.setSenha("dhsa432k");
			assinante.setNome("Maria Teresa Duarte");
			assinante.setApelido("Mah");
			
			//Persiste o assinante
			em.persist(assinante);			
			tx.commit();
			em.clear();
			
			Assert.assertNotNull(assinante.getId());
			Long uid = assinante.getId();
			
			/** Remover Assinante */
			
			//Busca o assinante
			assinante = em.find(Assinante.class, uid);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getListaAlbuns());
			Assert.assertNotNull(assinante.getListaGrupoAcesso());
			Assert.assertTrue(assinante.getListaAlbuns().isEmpty());
			Assert.assertTrue(assinante.getListaGrupoAcesso().isEmpty());
						
			//Remove o assinante
			tx.begin();			
			em.remove(assinante);
			tx.commit();
						
			//Confere que o assinante foi removido
			assinante = em.find(Assinante.class, uid);
			Assert.assertNull(assinante);

		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}
	}
	
	@Test
	public void testAtualizacaoAssinante(){
		
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
				
			/** Cadastra Assinante */
			
			//Constri um assinante
			Assinante assinante = new Assinante();
			assinante.setLogin("maria@terra.com.br");
			assinante.setSenha("dhsa432k");
			assinante.setNome("Maria Teresa Duarte");
			assinante.setApelido("Mah");
			
			//Persiste o assinante
			em.persist(assinante);			
			tx.commit();
			em.clear();
			
			/** Atualiza info assinante */
			
			assinante.setNome("Maria Teresa Duarte Silva");
			assinante.setApelido("Ma");
			
			tx.begin();
			//Assinante est detached, ento usamos merge
			assinante = em.merge(assinante);
			Long uid = assinante.getId();
			tx.commit();
			em.clear();
			
			//Busca novamente o assinante para conferir atualizao dos dados
			assinante = em.find(Assinante.class, uid);
			Assert.assertEquals("Maria Teresa Duarte Silva", assinante.getNome());
			Assert.assertEquals("Ma", assinante.getApelido());
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}			
	}
	
}
