package assinante;

import java.util.List;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.restlet.Client;
import org.restlet.data.Form;
import org.restlet.data.Protocol;
import org.restlet.data.Reference;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.Representation;

import br.usp.ime.pictware.application.assinante.service.ManterAssinanteService;
import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.AssinanteDAOHibernate;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.FiltroBuscaAssinante;

import com.thoughtworks.xstream.XStream;

import junit.framework.Assert;

public class TestAssinanteRest {

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}
	
	@Test
	public void listarAssinantes(){
		try{
			Client client = new Client(Protocol.HTTP);
			Response response =	client.get("http://localhost:8080/PictWare/rest/assinante/");
			String xml = response.getEntity().getText();
			Assert.assertNotNull(xml);
			Assert.assertFalse(xml.isEmpty());
			XStream xStream = new XStream();
			xStream.aliasType("assinantes", List.class);
			xStream.aliasType("assinante", Assinante.class);
			List<Assinante> listaAssinantes = (List<Assinante>)xStream.fromXML(xml);
			Assert.assertNotNull(listaAssinantes);
			
			for(Assinante assinante : listaAssinantes){
				Assert.assertNotNull(assinante.getId());
				Assert.assertNotNull(assinante.getNome());
				Assert.assertNotNull(assinante.getApelido());
				Assert.assertNotNull(assinante.getLogin());
			}
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}
	}
	
	@Test
	public void cadastrarAssinante(){
		
		ManterAssinanteService manterAssinanteService = new ManterAssinanteService();
		manterAssinanteService.setAssinanteDAO(new AssinanteDAOHibernate());
		
		try{
			
			String assinanteXML = 
			"<assinante>\n" +
			"  <nome>Roberta da Silva</nome>\n" +
			"  <apelido>Ro</apelido>\n" +
			"  <login>beta@hotmail.com</login>\n" +
			"  <senha>0102030405</senha>\n" +
			"</assinante>";
			
			String assinanteURI = "http://localhost:8080/PictWare/rest/assinante/"; 
			
			Form form = new Form();
			form.add("data", assinanteXML);
			Representation representation = form.getWebRepresentation();
			Client client = new Client(Protocol.HTTP);
			Response response = client.post(assinanteURI, representation);
			
			//Verifica se o requisio foi atendida com sucesso
			Assert.assertEquals(Status.SUCCESS_CREATED, response.getStatus());
			
			//Verifica se o xml voltou
			String xml = response.getEntity().getText();
			Assert.assertNotNull(xml);
			
			System.out.println(xml);
			
			//Verifica se foi gerado a URI para o novo recurso
			Reference identifier = response.getEntity().getIdentifier();
			Assert.assertNotNull(identifier);
			Assert.assertTrue(identifier.toString().startsWith(assinanteURI));
			Assert.assertTrue(identifier.toString().length() > assinanteURI.length());
			
			//Confirma o sucesso da insero
			Assert.assertTrue(response.getStatus().isSuccess());
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			//Exclui o assinante cadastrado
			FiltroBuscaAssinante filtroBuscaAssinante = new FiltroBuscaAssinante();
			filtroBuscaAssinante.setLogin("beta@hotmail.com");
			manterAssinanteService.excluirCadastroAssinante(filtroBuscaAssinante);
		}
	}
	
	@Test
	public void listarAssinante(){
		
		ManterAssinanteService manterAssinanteService = new ManterAssinanteService();
		manterAssinanteService.setAssinanteDAO(new AssinanteDAOHibernate());
		
		Assinante assinante = null;
		
		try{
			
			String login = "listarAssinante@teste.com.br";
			String senha = "123";
			String nomeAssinante = "nome";
			String apelido = "apelido";
			
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			
			Long id = assinante.getId();
			
			Client client = new Client(Protocol.HTTP);
			Response response =	client.get("http://localhost:8080/PictWare/rest/assinante/" + id);
			String xml = response.getEntity().getText();
			Assert.assertNotNull(xml);
			Assert.assertFalse(xml.isEmpty());
			XStream xStream = new XStream();
			xStream.aliasType("assinantes", List.class);
			xStream.aliasType("assinante", Assinante.class);
			List<Assinante> listaAssinantes = (List<Assinante>)xStream.fromXML(xml);
			Assert.assertNotNull(listaAssinantes);
			Assert.assertTrue(listaAssinantes.size() == 1);
			assinante = listaAssinantes.iterator().next();
			
			Assert.assertNotNull(assinante.getId());
			Assert.assertNotNull(assinante.getNome());
			Assert.assertNotNull(assinante.getApelido());
			Assert.assertNotNull(assinante.getLogin());
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			FiltroBuscaAssinante filtro = new FiltroBuscaAssinante();
			filtro.setLogin("listarAssinante@teste.com.br");
			manterAssinanteService.excluirCadastroAssinante(filtro);			
		}
	}
	
	@Test
	public void atualizarAssinante(){
		ManterAssinanteService manterAssinanteService = new ManterAssinanteService();
		manterAssinanteService.setAssinanteDAO(new AssinanteDAOHibernate());
		
		Assinante assinante = null;
		
		try{
		
			//Cadastra o assinante
			String login = "atualizarAssinante@teste.com.br";
			String senha = "123";
			String nomeAssinante = "nome";
			String apelido = "apelido";
			
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			
			//ID do assinante
			String uid = String.valueOf(assinante.getId());
			
			//Assinante com os dados atualizados
			String assinanteXML = 
				"<assinante>\n" +
				"  <nome>Assinante Atualizado</nome>\n" +
				"  <apelido>Atualizado</apelido>\n" +
				"  <senha>0102030405</senha>\n" +
				"</assinante>";
				
			String assinanteURI = "http://localhost:8080/PictWare/rest/assinante/" + uid; 
				
			Form form = new Form();
			form.add("data", assinanteXML);
			Representation representation = form.getWebRepresentation();
			Client client = new Client(Protocol.HTTP);
			Response response = client.put(assinanteURI, representation);
						
			//Verifica se o xml voltou
			String xml = response.getEntity().getText();
			Assert.assertNotNull(xml);
			
			//Confirma o sucesso da atualizao
			Assert.assertTrue(response.getStatus().isSuccess());
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			//Exclui o assinante cadastrado caso o teste falhe
			FiltroBuscaAssinante filtroBusca = new FiltroBuscaAssinante();
			filtroBusca.setLogin("atualizarAssinante@teste.com.br");
			manterAssinanteService.excluirCadastroAssinante(filtroBusca);
		}
	}

	@Test
	public void excluirAssinante(){
		
		ManterAssinanteService manterAssinanteService = new ManterAssinanteService();
		manterAssinanteService.setAssinanteDAO(new AssinanteDAOHibernate());
		
		try{
		
			//Cadastra o assinante
			String login = "excluirAssinante@teste.com.br";
			String senha = "123";
			String nomeAssinante = "nome";
			String apelido = "apelido";
			
			Assinante assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			
			String uid = String.valueOf(assinante.getId());
			
			Client client = new Client(Protocol.HTTP);
			Response response =	client.delete("http://localhost:8080/PictWare/rest/assinante/" + uid);
			
			//Verifica se o requisio foi atendida com sucesso
			Assert.assertEquals(Status.SUCCESS_NO_CONTENT, response.getStatus());
			
			//Confirma excluso			
			FiltroBuscaAssinante filtroBusca = new FiltroBuscaAssinante();
			filtroBusca.setUid(assinante.getId());
			assinante = manterAssinanteService.buscarAssinante(filtroBusca);
			Assert.assertNull(assinante);			
			
		}catch(Exception e){			
			e.printStackTrace();
			Assert.fail();
		}finally{
			//Exclui o assinante cadastrado caso o teste falhe
			FiltroBuscaAssinante filtroBusca = new FiltroBuscaAssinante();
			filtroBusca.setLogin("excluirAssinante@teste.com.br");
			manterAssinanteService.excluirCadastroAssinante(filtroBusca);
		}
	}
	
}
