package assinante;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;

import junit.framework.Assert;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import br.usp.ime.pictware.application.assinante.service.ManterAssinanteService;
import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.business.assinante.dao.AssinanteDAO;
import br.usp.ime.pictware.business.grupoacesso.bo.GrupoAcesso;
import br.usp.ime.pictware.business.grupoacesso.dao.GrupoAcessoDAO;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.AssinanteDAOHibernate;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.FiltroBuscaAssinante;
import br.usp.ime.pictware.infraestrutura.persistencia.grupoacesso.GrupoAcessoDAOHibernate;
import br.usp.ime.pictware.infraestrutura.persistencia.util.JPAUtil;

public class TestManterAssinanteService {
	
	private ManterAssinanteService manterAssinanteService;
	private Assinante assinante;
	
	@Before
	public void setUp() throws Exception {
		manterAssinanteService = new ManterAssinanteService();
		AssinanteDAO assinanteDAO = new AssinanteDAOHibernate();
		GrupoAcessoDAO grupoAcessoDAO = new GrupoAcessoDAOHibernate();
		manterAssinanteService.setAssinanteDAO(assinanteDAO);
		manterAssinanteService.setGrupoAcessoDAO(grupoAcessoDAO);
	}

	@After
	public void tearDown() throws Exception {
		
		//Garante que o assinante utilizado nos testes  excluido	
		EntityManager em = JPAUtil.getEntityManager();
		Session session = (Session)em.getDelegate();
		Transaction tx = session.beginTransaction();
		Criteria criteria = session.createCriteria(Assinante.class).
							add(Restrictions.eq("login", "x@x.com"));
		List<Assinante> listaAssinante = criteria.list();
		for (Assinante assinante : listaAssinante){
			em.remove(assinante);
		}		
		tx.commit();
	}
	
	@Test
	public void cadastrarAssinante(){
		
		EntityTransaction tx = null;
		
		try{
			String login = "x@x.com";
			String senha = "123";
			String nomeAssinante = "Joao da Silva";
			String apelido = "John";
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			Assert.assertNotNull(assinante.getLogin());
			Assert.assertNotNull(assinante.getSenha());
			Assert.assertNotNull(assinante.getNome());
			Assert.assertNotNull(assinante.getApelido());
			
			//Remove o assinante cadastrado
			
			EntityManager em = JPAUtil.getEntityManager();
			tx = em.getTransaction();
			tx.begin();			
			em.remove(assinante);
			tx.commit();
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx != null && tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}		
	}
	
	@Test
	public void cadastrarAssinanteComLoginDuplicado(){
		
		EntityTransaction tx = null;
		
		try{
			//Cadastra um assinante
			
			String login = "x@x.com";
			String senha = "12345";
			String nomeAssinante = "X da Silva";
			String apelido = "X";
			
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			Assert.assertNotNull(assinante.getLogin());
			Assert.assertNotNull(assinante.getSenha());
			Assert.assertNotNull(assinante.getNome());
			Assert.assertNotNull(assinante.getApelido());	
			
			Long uid = assinante.getId();
			
			//Tenta cadastrar um assinante com login duplicado
			
			senha = "123";
			nomeAssinante = "Maria Duarte";
			apelido = "Mah";
			
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNull(assinante);
			
			//Remove o assinante cadastrado
			
			EntityManager em = JPAUtil.getEntityManager();
			tx = em.getTransaction();
			tx.begin();
			assinante = em.find(Assinante.class, uid);
			Assert.assertNotNull(assinante);
			em.remove(assinante);
			tx.commit();
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx != null && tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}		
	}
	
	@Test
	public void excluirCadastroAssinante(){
		
		EntityTransaction tx = null;
		
		try{
			String login = "x@x.com";
			String senha = "123";
			String nomeAssinante = "Joao da Silva";
			String apelido = "John";
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			Assert.assertNotNull(assinante.getLogin());
			Assert.assertNotNull(assinante.getSenha());
			Assert.assertNotNull(assinante.getNome());
			Assert.assertNotNull(assinante.getApelido());
			
			Long uid = assinante.getId();
			
			//Remove o assinante cadastrado
			
			FiltroBuscaAssinante filtroBuscaAssinante = new FiltroBuscaAssinante();
			filtroBuscaAssinante.setUid(uid);
			manterAssinanteService.excluirCadastroAssinante(filtroBuscaAssinante);
			
			//Verifica se o cadastro foi removido
			
			EntityManager em = JPAUtil.getEntityManager();
			assinante = em.find(Assinante.class, uid);
			Assert.assertNull(assinante);
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx != null && tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}		
	}
	
	@Test
	public void cadastrarGrupoAcesso(){
		EntityTransaction tx = null;
		
		try{
			String login = "x@x.com";
			String senha = "123";
			String nomeAssinante = "Joao da Silva";
			String apelido = "John";
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			Assert.assertTrue(assinante.getListaGrupoAcesso().isEmpty());
			
			Long uid = assinante.getId();
			
			//Cadastra grupo de acesso
			
			String nomeGrupo = "Trabalho";
			manterAssinanteService.cadastrarGrupoAcesso(uid, nomeGrupo);
			
			//Busca assinante e confere cadastro do grupo de acesso
			
			EntityManager em = JPAUtil.getEntityManager();
			assinante = em.find(Assinante.class, uid);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getListaGrupoAcesso());
			Assert.assertEquals(1, assinante.getListaGrupoAcesso().size());
			
			GrupoAcesso grupoAcesso = assinante.getListaGrupoAcesso().iterator().next();
			Assert.assertNotNull(grupoAcesso.getId());
			Assert.assertEquals(nomeGrupo,grupoAcesso.getNome());
			Assert.assertNotNull(grupoAcesso.getListaMembros());
			Assert.assertTrue(grupoAcesso.getListaMembros().isEmpty());			
			
			//Remove assinante
			
			FiltroBuscaAssinante filtroBuscaAssinante = new FiltroBuscaAssinante();
			filtroBuscaAssinante.setUid(uid);
			manterAssinanteService.excluirCadastroAssinante(filtroBuscaAssinante);
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx != null && tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}	
	}
	
	@Test
	public void cadastrarGrupoAcessoDuplicado(){
		EntityTransaction tx = null;
		
		try{
			String login = "x@x.com";
			String senha = "123";
			String nomeAssinante = "Joao da Silva";
			String apelido = "John";
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			Assert.assertTrue(assinante.getListaGrupoAcesso().isEmpty());
			
			Long uid = assinante.getId();
			
			//Cadastra grupo de acesso
			
			String nomeGrupo = "Trabalho";
			manterAssinanteService.cadastrarGrupoAcesso(uid, nomeGrupo);
			
			//Cadastra novamente o grupo
			manterAssinanteService.cadastrarGrupoAcesso(uid, nomeGrupo);
			
			//Busca assinante e confere cadastro do grupo de acesso
			
			EntityManager em = JPAUtil.getEntityManager();
			assinante = em.find(Assinante.class, uid);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getListaGrupoAcesso());
			
			//Confere se apenas um grupo foi cadastrado
			Assert.assertEquals(1, assinante.getListaGrupoAcesso().size());
			
			//Remove assinante
			
			FiltroBuscaAssinante filtroBuscaAssinante = new FiltroBuscaAssinante();
			filtroBuscaAssinante.setUid(uid);
			manterAssinanteService.excluirCadastroAssinante(filtroBuscaAssinante);
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx != null && tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}	
	}
	
	@Test
	public void buscarGrupoAcesso(){
		EntityTransaction tx = null;
		
		try{
			String login = "x@x.com";
			String senha = "123";
			String nomeAssinante = "Joao da Silva";
			String apelido = "John";
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			Assert.assertTrue(assinante.getListaGrupoAcesso().isEmpty());
			
			Long uid = assinante.getId();
			
			//Cadastra grupo de acesso
			
			String nomeGrupo = "Trabalho";
			assinante = manterAssinanteService.cadastrarGrupoAcesso(uid, nomeGrupo);
			Long grupoId = assinante.getListaGrupoAcesso().iterator().next().getId();
			Assert.assertNotNull(grupoId);
			
			//Busca grupo de acesso
			
			GrupoAcesso grupoAcesso = manterAssinanteService.buscarGrupo(grupoId);
			Assert.assertNotNull(grupoAcesso);
			Assert.assertNotNull(grupoAcesso.getId());
			Assert.assertNotNull(grupoAcesso.getNome());
			Assert.assertNotNull(grupoAcesso.getListaMembros());
			Assert.assertTrue(grupoAcesso.getListaMembros().isEmpty());
			
			//Remove assinante (e grupo de acesso)
			
			FiltroBuscaAssinante filtroBuscaAssinante = new FiltroBuscaAssinante();
			filtroBuscaAssinante.setUid(uid);
			manterAssinanteService.excluirCadastroAssinante(filtroBuscaAssinante);
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx != null && tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}	
	}
	
	@Test
	public void buscarGrupoAcessoInexistente(){
		EntityTransaction tx = null;
		
		try{
			String login = "x@x.com";
			String senha = "123";
			String nomeAssinante = "Joao da Silva";
			String apelido = "John";
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			Assert.assertTrue(assinante.getListaGrupoAcesso().isEmpty());
			
			Long uid = assinante.getId();
					
			//Busca grupo de acesso inexistente
			
			Long grupoId = new Long(999);
			GrupoAcesso grupoAcesso = manterAssinanteService.buscarGrupo(grupoId);
			Assert.assertNull(grupoAcesso);
			
			//Remove assinante
			
			FiltroBuscaAssinante filtroBuscaAssinante = new FiltroBuscaAssinante();
			filtroBuscaAssinante.setUid(uid);
			manterAssinanteService.excluirCadastroAssinante(filtroBuscaAssinante);
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx != null && tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}	
	}
	
	@Test
	public void excluirGrupoAcesso(){
		EntityTransaction tx = null;
		
		try{
			String login = "x@x.com";
			String senha = "123";
			String nomeAssinante = "Joao da Silva";
			String apelido = "John";
			assinante = manterAssinanteService.cadastrarAssinante(login, senha, nomeAssinante, apelido);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getId());
			Assert.assertTrue(assinante.getListaGrupoAcesso().isEmpty());
			
			Long uid = assinante.getId();
			
			//Cadastra grupo de acesso
			
			String nomeGrupo = "Trabalho";
			assinante = manterAssinanteService.cadastrarGrupoAcesso(uid, nomeGrupo);
			Long grupoId = assinante.getListaGrupoAcesso().iterator().next().getId();
			Assert.assertNotNull(grupoId);
			
			//Remove grupo de acesso

			manterAssinanteService.excluirGrupoAcesso(uid, grupoId);
			
			//Busca assinante e confere que grupo foi removido
						
			EntityManager em = JPAUtil.getEntityManager();
			assinante = em.find(Assinante.class, uid);
			Assert.assertNotNull(assinante);
			Assert.assertNotNull(assinante.getListaGrupoAcesso());
			Assert.assertTrue(assinante.getListaGrupoAcesso().isEmpty());
			
			//Remove assinante
			
			FiltroBuscaAssinante filtroBuscaAssinante = new FiltroBuscaAssinante();
			filtroBuscaAssinante.setUid(uid);
			manterAssinanteService.excluirCadastroAssinante(filtroBuscaAssinante);
			
		}catch(Exception e){
			e.printStackTrace();
			Assert.fail();
		}finally{
			if (tx != null && tx.isActive()){
				tx.rollback();
				Assert.fail();
			}			
		}	
	}
}
