package br.usp.ime.pictware.application.assinante.service;

import java.util.List;

import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.business.assinante.dao.AssinanteDAO;
import br.usp.ime.pictware.business.grupoacesso.bo.GrupoAcesso;
import br.usp.ime.pictware.business.grupoacesso.dao.GrupoAcessoDAO;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.FiltroBuscaAssinante;

public class ManterAssinanteService {

	private AssinanteDAO assinanteDAO;
	private GrupoAcessoDAO grupoAcessoDAO;
	
	public ManterAssinanteService(){
		
	}
	
	public AssinanteDAO getAssinanteDAO(){
		return assinanteDAO;
	}
	
	public void setAssinanteDAO(AssinanteDAO assinanteDAO){
		this.assinanteDAO = assinanteDAO;
	}
	
	public GrupoAcessoDAO getGrupoAcessoDAO() {
		return grupoAcessoDAO;
	}

	public void setGrupoAcessoDAO(GrupoAcessoDAO grupoAcessoDAO) {
		this.grupoAcessoDAO = grupoAcessoDAO;
	}

	
	public Assinante buscarAssinante(FiltroBuscaAssinante criterioBusca){
		Assinante assinante = getAssinanteDAO().buscarAssinante(criterioBusca);
		return assinante;
	}
	
	public List<Assinante> buscarAssinantes(){
		return getAssinanteDAO().buscarAssinantes();
	}
	
	public Assinante atualizarInfoAssinante(Long uid, String senha, String nomeAssinante,
			String apelido) {
		
		FiltroBuscaAssinante filtroBusca = new FiltroBuscaAssinante();
		filtroBusca.setUid(uid);
		Assinante assinante = getAssinanteDAO().buscarAssinante(filtroBusca);
		assinante.setNome(nomeAssinante);
		assinante.setApelido(apelido);
		assinante.setSenha(senha);
		return getAssinanteDAO().atualizarAssinante(assinante);
	}

	public Assinante cadastrarAssinante(String login, String senha,
			String nomeAssinante, String apelido) {
		
		Assinante assinante = null;
		
		if (getAssinanteDAO().existeAssinante(login)){
			//throw exception
		}
		else{
			assinante = new Assinante();
			assinante.setLogin(login);
			assinante.setSenha(senha);
			assinante.setNome(nomeAssinante);
			assinante.setApelido(apelido);		
			
			getAssinanteDAO().incluirAssinante(assinante);
		}
						
		return assinante;		
	}
	
	public Assinante cadastrarGrupoAcesso(Long ownerId, String nomeGrupo){
		
		GrupoAcesso grupoAcesso = new GrupoAcesso();
		grupoAcesso.setNome(nomeGrupo);

		FiltroBuscaAssinante filtroBusca = new FiltroBuscaAssinante();
		filtroBusca.setUid(ownerId);
		Assinante owner = getAssinanteDAO().buscarAssinante(filtroBusca);
		
		GrupoAcessoDAO grupoAcessoDAO = getGrupoAcessoDAO();
		boolean existeGrupoMesmoNome = grupoAcessoDAO.existeGrupoAcesso(ownerId,nomeGrupo);

		if (!existeGrupoMesmoNome){
			owner.addGrupoAcesso(grupoAcesso);
			getAssinanteDAO().atualizarAssinante(owner);
		}
		else{
			//TODO: tratar grupo acesso duplicado
			System.out.println("Grupo de acesso duplicado");
		}
		
		return owner;
	}

	public void excluirCadastroAssinante(FiltroBuscaAssinante filtroBusca){
		Assinante assinante = getAssinanteDAO().buscarAssinante(filtroBusca);
		if (assinante != null){
			getAssinanteDAO().excluirAssinante(assinante);
		}
	}
	
	public Assinante excluirGrupoAcesso(Long ownerId, Long grupoId) {
		
		FiltroBuscaAssinante filtroBusca = new FiltroBuscaAssinante();
		filtroBusca.setUid(ownerId);
		Assinante owner = getAssinanteDAO().buscarAssinante(filtroBusca);
		GrupoAcesso grupoAcesso = getGrupoAcessoDAO().buscarGrupoAcesso(grupoId);
		owner.getListaGrupoAcesso().remove(grupoAcesso);
		getAssinanteDAO().atualizarAssinante(owner);
		return owner;		
	}

	public GrupoAcesso buscarGrupo(Long grupoId) {
		GrupoAcesso grupoAcesso = getGrupoAcessoDAO().buscarGrupoAcesso(grupoId);
		return grupoAcesso;
	}
	
}
