package br.usp.ime.pictware.aspects.logging;

import java.util.Enumeration;

import org.apache.log4j.Appender;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.util.ClassUtils;

@Aspect
public class LoggingAspect {
	
	@Pointcut("execution(public * *(..))")
	private void anyPublicOperation() {}

	/**
	 * A join point is in the service layer if the method is defined
	 * in a type in the com.xyz.someapp.service package or any sub-package
	 * under that.
	 */
	@Pointcut("within(br.usp.ime.pictware.business..*)")
	private void inBusinessLayer() {
		
	}
	
	/**
	 * A join point is in the service layer if the method is defined
	 * in a type in the com.xyz.someapp.service package or any sub-package
	 * under that.
	 */
	@Pointcut("within(br.usp.ime.pictware.infraestrutura.persistencia..*)")
	private void inDataAccessLayer() {
		
	}

	@Pointcut("anyPublicOperation() && (inBusinessLayer() || inDataAccessLayer())")
	private void loggableMethod(){
		
	}
	
	@Around("loggableMethod()")
	public Object doLogging(ProceedingJoinPoint pjp) throws Throwable{
		//Log Data
		Class clazz = pjp.getTarget().getClass();
		String methodSignature = pjp.getSignature().toLongString();
		
		//The Logger
		Logger logger = Logger.getLogger(clazz);
		ConsoleAppender appender = new ConsoleAppender(new PatternLayout(PatternLayout.TTCC_CONVERSION_PATTERN));
		appender.setTarget(ConsoleAppender.SYSTEM_OUT);
		logger.addAppender(appender);
		logger.setLevel(Level.ALL);
		
		//Logging entry in method
		logger.info("Entering method " + methodSignature);
		
		//Execute the underlying method
		Object retVal = pjp.proceed();
		
		//Logging exit in method
		logger.info("Exiting method " + methodSignature);
		
		return retVal;
	}
	
}
