package br.usp.ime.pictware.infraestrutura.persistencia.album;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;

import br.usp.ime.pictware.business.album.bo.Album;
import br.usp.ime.pictware.business.album.bo.Foto;
import br.usp.ime.pictware.business.album.dao.AlbumDAO;
import br.usp.ime.pictware.infraestrutura.persistencia.util.JPAUtil;

public class AlbumDAOHibernate implements AlbumDAO{
		
	/* (non-Javadoc)
	 * @see br.usp.ime.pictware.infraestrutura.persistencia.album.AlbumDAO#incluirAlbum(br.usp.ime.pictware.business.album.bo.Album)
	 */
	public void incluirAlbum(Album album) {
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
			
			em.persist(album);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}
	}
	
	/* (non-Javadoc)
	 * @see br.usp.ime.pictware.infraestrutura.persistencia.album.AlbumDAO#incluirFoto(br.usp.ime.pictware.business.album.bo.Foto)
	 */
	public void incluirFoto(Foto foto) {
		
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
			
			em.persist(foto);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}
	}

	/* (non-Javadoc)
	 * @see br.usp.ime.pictware.infraestrutura.persistencia.album.AlbumDAO#excluirAlbum(br.usp.ime.pictware.business.album.bo.Album)
	 */
	public void excluirAlbum(Album album) {
		
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
			
			em.remove(album);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}
		
	}

	/* (non-Javadoc)
	 * @see br.usp.ime.pictware.infraestrutura.persistencia.album.AlbumDAO#excluirFoto(br.usp.ime.pictware.business.album.bo.Foto)
	 */
	public void excluirFoto(Foto foto) {
		
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
			
			em.remove(foto);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}
	}
	
	/* (non-Javadoc)
	 * @see br.usp.ime.pictware.infraestrutura.persistencia.album.AlbumDAO#atualizarAlbum(br.usp.ime.pictware.business.album.bo.Album)
	 */
	public Album atualizarAlbum(Album album){
		
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
			
			album = em.merge(album);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}
		
		return album;
	}

	/* (non-Javadoc)
	 * @see br.usp.ime.pictware.infraestrutura.persistencia.album.AlbumDAO#buscarAlbum(java.lang.Long)
	 */
	public Album buscarAlbum(Long albumId) {
		
		EntityManager em = JPAUtil.getEntityManager();	
		return em.find(Album.class,albumId);
	}

	/* (non-Javadoc)
	 * @see br.usp.ime.pictware.infraestrutura.persistencia.album.AlbumDAO#buscarFoto(java.lang.Long)
	 */
	public Foto buscarFoto(Long fotoId) {
		
		EntityManager em = JPAUtil.getEntityManager();	
		return em.find(Foto.class,fotoId);
	}

	public Foto atualizarFoto(Foto foto) {
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
			
			foto = em.merge(foto);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}		
		
		return foto;
	}
}