package br.usp.ime.pictware.infraestrutura.persistencia.assinante;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.business.assinante.dao.AssinanteDAO;
import br.usp.ime.pictware.infraestrutura.persistencia.util.JPAUtil;

public class AssinanteDAOHibernate implements AssinanteDAO{
			
	public Assinante buscarAssinante(FiltroBuscaAssinante filtro) {
			 			
		EntityManager em = JPAUtil.getEntityManager();
		Session session = (Session)em.getDelegate();
				
		Criteria criteria = session.createCriteria(Assinante.class);
		Assinante exemplo = new Assinante();
		exemplo.setLogin(filtro.getLogin());
		exemplo.setSenha(filtro.getSenha());
		Example exampleRestriction = Example.create(exemplo);
		criteria = criteria.add(exampleRestriction);
		
		if (filtro.getUid() != null){
			criteria = criteria.add(Restrictions.eq("id",filtro.getUid()));
		}
			
		return (Assinante)criteria.uniqueResult();
	}
	
	public Assinante atualizarAssinante(Assinante assinante){
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
			
			assinante = em.merge(assinante);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}
		
		return assinante;
	}

	public void incluirAssinante(Assinante assinante) {
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
				
			em.persist(assinante);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}		
	}

	public void excluirAssinante(Assinante assinante) {
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
			
			em.remove(assinante);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}
	}
	
	public boolean existeAssinante(String login){
		EntityManager em = JPAUtil.getEntityManager();
		Session session = (Session)em.getDelegate();
				
		Criteria criteria = session.createCriteria(Assinante.class).
			setProjection(Projections.rowCount()).
			add(Restrictions.eq("login", login));
		Integer numAssinantes = (Integer)criteria.uniqueResult();
		return (numAssinantes>0);
	}

	public List<Assinante> buscarAssinantes() {
		EntityManager em = JPAUtil.getEntityManager();
		String jpql = "SELECT a FROM Assinante a";
		Query query = em.createQuery(jpql);
		return query.getResultList();
	}

}
