package br.usp.ime.pictware.infraestrutura.persistencia.grupoacesso;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;

import org.hibernate.Query;
import org.hibernate.Session;

import br.usp.ime.pictware.business.grupoacesso.bo.GrupoAcesso;
import br.usp.ime.pictware.business.grupoacesso.bo.MembroGrupo;
import br.usp.ime.pictware.business.grupoacesso.dao.GrupoAcessoDAO;
import br.usp.ime.pictware.infraestrutura.persistencia.util.JPAUtil;

public class GrupoAcessoDAOHibernate implements GrupoAcessoDAO{

	public GrupoAcesso atualizarGrupoAcesso(GrupoAcesso grupoAcesso) {
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
			
			grupoAcesso = em.merge(grupoAcesso);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}
		
		return grupoAcesso;		
	}

	public boolean existeGrupoAcesso(Long ownerId, String nomeGrupo) {
		EntityManager em = JPAUtil.getEntityManager();
		Session session = (Session)em.getDelegate();
				
		Query query = session.createQuery("select count(grupoAcesso) from Assinante assinante join assinante.listaGrupoAcesso grupoAcesso where assinante.id=:ownerId and grupoAcesso.nome=:nomeGrupo");
		query.setLong("ownerId", ownerId);
		query.setString("nomeGrupo", nomeGrupo);
		Long numGruposAcesso = (Long)query.list().iterator().next();
		return (numGruposAcesso>0);
	}

	public GrupoAcesso buscarGrupoAcesso(Long grupoAcessoId) {
		EntityManager em = JPAUtil.getEntityManager();	
		return em.find(GrupoAcesso.class,grupoAcessoId);		
	}

	public MembroGrupo buscarMembro(Long membroId) {
		EntityManager em = JPAUtil.getEntityManager();	
		return em.find(MembroGrupo.class,membroId);
	}

	public void incluirMembro(MembroGrupo membro) {
		EntityManager em = JPAUtil.getEntityManager();
		EntityTransaction tx = em.getTransaction();
		
		try{			
			//Begin transaction
			tx.begin();
				
			em.persist(membro);
			
			//Commit transaction
			tx.commit();
		}finally{
			if (tx.isActive()) tx.rollback();
		}		
	}
	
}
