package br.usp.ime.pictware.web.action.manteralbum;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;


import br.usp.ime.pictware.application.album.service.ManterAlbumService;
import br.usp.ime.pictware.business.album.bo.Foto;
import br.usp.ime.pictware.web.framework.action.Action;
import br.usp.ime.pictware.web.framework.action.ActionForward;
import br.usp.ime.pictware.web.framework.action.ActionMapping;

public class InserirFotoAction implements Action {

	private static final Logger LOG = Logger.getLogger(InserirFotoAction.class.getName());
	
	public ActionForward execute(ActionMapping actionMapping,
			HttpServletRequest request, HttpServletResponse response) {

		try{
			
			WebApplicationContext ctx = WebApplicationContextUtils.getRequiredWebApplicationContext(request.getSession().getServletContext());
			ManterAlbumService manterAlbumService = (ManterAlbumService)ctx.getBean("albumService");
			
			FileItemFactory factory = new DiskFileItemFactory();

			// Cria um handler para upload de arquivos
			ServletFileUpload upload = new ServletFileUpload(factory);
			
			// Faz o parse do request
			List<FileItem> listaFileItem = upload.parseRequest(request);
			
			byte[] bytesArquivo = null;
			String nomeArquivo = null;
			Long tamanhoArquivo = null;
			String tipoArquivo = null;
			String legendaFoto = null;			
			
			for (FileItem item : listaFileItem){
				
				//Processa fields
			    if (item.isFormField()) {
			    	
			    	if (item.getFieldName().equals("legenda")){
			    		legendaFoto = item.getString();	
			    	}			    	
			    				    	
			    //Processa arquivos
			    } else {
			    	bytesArquivo = item.get();
			    	nomeArquivo = item.getName();			        			  
			        tamanhoArquivo = item.getSize();
			        tipoArquivo = item.getContentType();
			    }			    
			}
			
			//Resgata assinante da sesso
			Long albumId = Long.parseLong(request.getParameter("albumId"));						
			Foto foto = manterAlbumService.incluirFoto(albumId,bytesArquivo,nomeArquivo,tamanhoArquivo,tipoArquivo,legendaFoto);
			writeToFile(request,foto,bytesArquivo);
			
			request.setAttribute("album", foto.getAlbum());
			
			return actionMapping.findForward("foto-inserida");
		}catch(FileUploadException e){
			LOG.log(Level.SEVERE, "No foi possvel realizar upload da foto",e);
			e.printStackTrace();
		}catch(Exception e){
			e.printStackTrace();
		}
		return actionMapping.findForward("foto-inserida");	
	}
	
	private void writeToFile(HttpServletRequest request, Foto foto, byte[] bytesArquivo){
		try{
			
			Map<String,String> map = new HashMap<String, String>();
			map.put("image/jpeg", ".jpg");
			
			Long userId = (Long)request.getSession().getAttribute("uid");
			String path = request.getSession().getServletContext().getRealPath("/userpics");
			FileOutputStream fos = new FileOutputStream(path + "/" + userId + "_" + foto.getId() + map.get(foto.getTipoArquivo()));
	        fos.write(bytesArquivo);
	        fos.close();
		}catch(IOException e){
			
		}
	}
}
