package br.usp.ime.pictware.web.webservice;

import java.util.ArrayList;
import java.util.List;

import org.restlet.Context;
import org.restlet.data.Form;
import org.restlet.data.MediaType;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.StringRepresentation;
import org.restlet.resource.Variant;

import br.ime.usp.pictware.util.AssinanteXMLConverter;
import br.usp.ime.pictware.application.assinante.service.ManterAssinanteService;
import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.AssinanteDAOHibernate;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.FiltroBuscaAssinante;

public class AssinanteResource extends Resource {

	private ManterAssinanteService manterAssinanteService;
	private String uid;
	
	public AssinanteResource(Context context, Request request, Response response){
		super(context,request,response);
		this.getVariants().add(new Variant(MediaType.TEXT_XML));
				
		this.manterAssinanteService = new ManterAssinanteService();
		manterAssinanteService.setAssinanteDAO(new AssinanteDAOHibernate());
		
		this.uid = (String)request.getAttributes().get("assinante_id");
	}
	
	@Override
	public Representation getRepresentation(Variant variant){
		FiltroBuscaAssinante filtroBusca = new FiltroBuscaAssinante();
		filtroBusca.setUid(Long.parseLong(uid));
		Assinante assinante = manterAssinanteService.buscarAssinante(filtroBusca);
		List<Assinante> listaAssinantes = new ArrayList<Assinante>();
		listaAssinantes.add(assinante);
		String xml = AssinanteXMLConverter.assinantesToXML(listaAssinantes);
		StringRepresentation stringRepresentation = new StringRepresentation(xml);
		return stringRepresentation;
	}
	
	@Override
	public boolean allowPut() {
		return true;
	}
	
	@Override
	public void put(Representation representation) {
	
		Form form = new Form(representation);
		String assinanteXML = form.getFirstValue("data");
		Representation rep = null;
		try {
			Assinante assinante = AssinanteXMLConverter.assinanteFromXML(assinanteXML);
			
			//ID do assinante vem do request
			assinante = manterAssinanteService.atualizarInfoAssinante(
					Long.parseLong(uid),
					assinante.getSenha(),
					assinante.getNome(),
					assinante.getApelido());
			getResponse().setStatus(Status.SUCCESS_OK);
			
			//XML retornado  o mesmo do request
			rep = new StringRepresentation(assinanteXML, MediaType.TEXT_XML);
			rep.setIdentifier(getRequest().getResourceRef().getIdentifier() + uid);
		} catch (Throwable thr) {
			getResponse().setStatus(Status.SERVER_ERROR_INTERNAL);
			rep = new StringRepresentation("Erro ao atualizar cadastro do assinante",MediaType.TEXT_PLAIN);
		}
		getResponse().setEntity(rep);
	}
	
	@Override
	public boolean allowDelete() {
		return true;
	}
	
	@Override
	public void delete() {
		try{
			FiltroBuscaAssinante filtroBusca = new FiltroBuscaAssinante();
			//ID do assinante vem do request
			filtroBusca.setUid(Long.parseLong(uid));
			manterAssinanteService.excluirCadastroAssinante(filtroBusca);
			getResponse().setStatus(Status.SUCCESS_NO_CONTENT);
		}catch(Throwable thr){
			getResponse().setStatus(Status.SERVER_ERROR_INTERNAL);
			getResponse().setEntity(new StringRepresentation("Erro ao atualizar cadastro do assinante",MediaType.TEXT_PLAIN));
		}
	}
	
}
