package br.usp.ime.pictware.web.webservice;

import java.util.List;

import org.restlet.Context;
import org.restlet.data.Form;
import org.restlet.data.MediaType;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.resource.Representation;
import org.restlet.resource.Resource;
import org.restlet.resource.StringRepresentation;
import org.restlet.resource.Variant;

import br.ime.usp.pictware.util.AssinanteXMLConverter;
import br.usp.ime.pictware.application.assinante.service.ManterAssinanteService;
import br.usp.ime.pictware.business.assinante.bo.Assinante;
import br.usp.ime.pictware.infraestrutura.persistencia.assinante.AssinanteDAOHibernate;

public class AssinantesResource extends Resource {

	private ManterAssinanteService manterAssinanteService;
	
	public AssinantesResource(Context context, Request request, Response response){
		super(context,request,response);
		this.getVariants().add(new Variant(MediaType.TEXT_XML));
		
		this.manterAssinanteService = new ManterAssinanteService();
		manterAssinanteService.setAssinanteDAO(new AssinanteDAOHibernate());
	}
	
	@Override
	public Representation getRepresentation(Variant variant){
		
		List<Assinante> listaAssinantes = manterAssinanteService.buscarAssinantes();
		String xml = AssinanteXMLConverter.assinantesToXML(listaAssinantes);	
		StringRepresentation stringRepresentation = new StringRepresentation(xml);
		return stringRepresentation;
	}
	
	@Override
	public boolean allowPost() {
		return true;
	}
	
	@Override
	public void post(Representation representation) {
		Form form = new Form(representation);
		String assinanteXML = form.getFirstValue("data");
		Representation rep = null;
		try {
			Assinante assinante = AssinanteXMLConverter.assinanteFromXML(assinanteXML);
			assinante = manterAssinanteService.cadastrarAssinante(
					assinante.getLogin(), 
					assinante.getSenha(),
					assinante.getNome(),
					assinante.getApelido());
			getResponse().setStatus(Status.SUCCESS_CREATED);
			
			//XML retornado  o mesmo do request
			rep = new StringRepresentation(assinanteXML, MediaType.TEXT_XML);
			rep.setIdentifier(getRequest().getResourceRef().getIdentifier() + assinante.getId());
		} catch (Throwable thr) {
			getResponse().setStatus(Status.SERVER_ERROR_INTERNAL);
			rep = new StringRepresentation("Erro ao cadastrar assinante",MediaType.TEXT_PLAIN);
		}
		getResponse().setEntity(rep);
	}
	
	
}
